this.name					= "gates_jumpGate";
this.author					= "Thargoid";
this.copyright				= "Creative Commons: attribution, non-commercial, sharealike with clauses - see readme.txt";
this.description			= "Script for Thargoid jumpgates";
this.version				= "1.12";


function gates_allGates(entity) {return entity.isShip && entity != this.ship && entity.hasRole("gates_jumpGate")}; // all gates except this one

function gates_clients(entity) {return entity.isShip && entity.primaryRole != "gates_gateGuard" && !entity.isCloaked && !entity.isWeapon && !entity.isRock && !entity.isCargo && (!entity.hasOwnProperty("gates_receivingGate")|| entity.gates_receivingGate == null)}; // ships that can use jumpgates (exclude spawned defenders, missiles, mines, asteroids and cargo pods

function gates_oldClients(entity) {return entity.isShip && entity.hasOwnProperty("gates_receivingGate") && entity.gates_receivingGate != null && entity.position.distanceTo(this.ship.position) > 500 && entity.position.distanceTo(this.ship.position) < 5000}; // ships which have come through this gate and are now between 500m and 5km from it

function gates_outboundNPCs(entity) {return entity.isShip && entity.AI == "exitingTraderAI.plist" && entity.AIState == "HEAD_AWAY_FROM_PLANET" && (!entity.escorts || entity.escorts.length == 0) && system.mainStation && entity.position.distanceTo(system.mainStation.position) < 51200};

function gates_inboundNPCs(entity) {return entity.isShip && entity.AI == "route1traderAI.plist" && entity.AIState == "HEAD_FOR_PLANET" && (!entity.escorts || entity.escorts.length == 0) && entity.position.distanceTo([0,0,0]) < 51200};


this.shipSpawned = function()
	{
	this.ship.scannerDisplayColor1 = "greenColor";
	this.ship.scannerDisplayColor2 = "whiteColor";
	this.playerGreeted = null;
	this.allowGreeting = true;
	this.gatesOnline = true;
	this.ship.orientation = system.mainStation.orientation; // point them all in the same direction, along universal Z.
	this.defenderCount = 10; // max number of defenders that can be launched overall by the gate if attacked
	
	if(this.scanTimer)
		{
		this.scanTimer.start();
		}
	else
		{
		this.scanTimer = new Timer(this, this.shipApproach, 0, 1.0); // scan for jumping ships every 1 second - too long/short?
		}

	if(this.npcTimer)
		{
		this.npcTimer.start();
		}
	else
		{
		this.npcTimer = new Timer(this, this.npcTraders, 0, 180); // scan for NPC traders who may want to use the gates every 3 minutes
		}

	}

this.playerWillEnterWitchspace = this.shipDied = function()
	{
	if(this.scanTimer)
		{
		this.scanTimer.stop();
		}

	if(this.npcTimer)
		{
		this.npcTimer.stop();
		}
	}

this.shipApproach = function()
	{	
	if(!player.ship || !player.ship.isValid)
		{
		this.shipDied();
		return;
		}
	
	if(this.ship.position.distanceTo(player.ship.position) > 25600 && this.allowGreeting == null)
		{
		this.allowGreeting = true;
		}
	
	var pastClients = system.filteredEntities(this, gates_oldClients, this.ship, 5000) // if any old clients are around, strip their client status
	if(pastClients.length > 0)
		{
		for(var i = 0; i < pastClients.length; i++)
			{
			pastClients[i].gates_receivingGate = null;
			if(pastClients[i].isPlayer) 	
				{
				player.consoleMessage("Jump gate cleared.", 6);
				this.allowGreeting = null;
				}
			}
		}
	
	var queue = system.filteredEntities(this, gates_clients, this.ship, 250); // if there is no-one wanting to jump, end the test
	if(queue.length == 0)
		{
		return;
		}
	
	var gates = system.filteredEntities(this, gates_allGates, this.ship); // check whether there is a second gate to receive the jumper
	if(gates.length == 0) 
		{
		if(queue[0].isPlayer && this.gatesOnline == true) 
			{
			player.consoleMessage("Jump gate off-line.", 6);
			this.gatesOnline = null;
			}
		return;
		}
	else
		{
		this.gatesOnline = true;
		}
	
	if(queue[0].hasOwnProperty("gates_receivingGate") && queue[0].gates_receivingGate == this.ship) // if the jumper has just arrived, end the test
		{
		return;
		}

	if(queue[0].AI == "gates_useGateAI.plist")
		{
		queue[0].reactToAIMessage("JUMPED");
		queue[0].gates_receivingGate = gates[0]; // note the gate that is about to receive the jumper
		queue[0].position = gates[0].position; // move the jumper to the receiving gate
		queue[0].orientation = gates[0].orientation; // re-orient the jumper to point out of the gate
		return;
		}
	
	if(queue[0].isPlayer)
		{
		if(player.credits < 251)
			{
			player.consoleMessage("Insufficient credits to perform jump.", 6);
			return;
			}
		else
			{
			player.consoleMessage("Jump gate activated.", 6);
			player.credits -=250;
			this.playerGreeted = null;
			queue[0].gates_receivingGate = gates[0]; // note the gate that is about to receive the player
			queue[0].position = gates[0].position; // move the player to the receiving gate
			queue[0].orientation = gates[0].orientation; // re-orient the player to point out of the gate
			}
		}
	}

this.greetPlayer = function()
	{
	if(!player.ship.isCloaked && (!player.ship.hasOwnProperty("gates_receivingGate") || player.ship.gates_receivingGate == null) && this.playerGreeted != true && this.allowGreeting == true)
		{
		player.consoleMessage("Welcome Commander, only 250Cr to use the jump gate system.", 6);
		this.playerGreeted = true;
		}
	}

this.findGates = function()
	{
	var gates = system.filteredEntities(this, gates_allGates, this.ship);	
	if(gates.length == 0)
		{
		this.ship.reactToAIMessage("NO_GATE");
		this.ship.scannerDisplayColor1 = "greenColor";
		this.ship.scannerDisplayColor2 = "orangeColor";
		}
	else
		{
		this.ship.reactToAIMessage("GATE_FOUND");
		this.ship.scannerDisplayColor1 = "greenColor";
		this.ship.scannerDisplayColor2 = "whiteColor";
		}
	};
	
this.underAttack = function()
	{
	if(this.defenderCount > 0 && worldScripts["gates_masterScript"] && worldScripts["gates_masterScript"].defenderAllowed())
		{
		var defender = this.ship.spawnOne("police");
		defender.position = this.ship.position;
		defender.orientation = this.ship.orientation;
		this.defenderCount -= 1;
		}
	}

this.npcTraders = function()
	{
	var outTraders = system.filteredEntities(this, gates_outboundNPCs); // list of outbound traders
	var inTraders = system.filteredEntities(this, gates_inboundNPCs); // list of inbound traders
	var gates = system.shipsWithRole("gates_jumpGate");

	if(gates.length < 2)
		{
		return;
		}

	if(inTraders.length > 0)
		{
		for(var i = 0; i < inTraders.length; i++)
			{
			if(Math.random() < 0.2 && gates.length > 0)
				{
				inTraders[i].target = gates[0];
				inTraders[i].script.checkGateDistance = this.checkGateDistance;
				inTraders[i].setAI("gates_useGateAI.plist");
				}
			}
		}

	if(outTraders.length > 0)
		{
		for(var i = 0; i < outTraders.length; i++)
			{
			if(Math.random() < 0.2 && gates.length > 0)
				{
				outTraders[i].target = gates[0];
				outTraders[i].script.checkGateDistance = this.checkGateDistance;
				outTraders[i].setAI("gates_useGateAI.plist");
				}
			}
		}
	}

// The function below is assigned to NPC ships using the jump gates

this.checkGateDistance = function()
	{	
	if(this.ship.position.distanceTo(this.ship.target.position) > 51200)
		{
		this.ship.reactToAIMessage("NEXT_GATE")
		}
	}
